@@ -35,8 +35,7 @@ def require_programs():
 
 
 def ask_for_username(args):
-    """
-    Ask for a reasonable username for the non-root user.
+    """Ask for a reasonable username for the non-root user.
 
     :returns: the username
     """
@@ -52,13 +51,12 @@ def ask_for_username(args):
 
 
 def ask_for_work_path(args):
-    """
-    Ask for the work path, until we can create it (when it does not exist) and
-    write into it.
+    """Ask for the work path, until we can create it (when it does not exist) and write into it.
+
     :returns: (path, exists)
-              * path: is the full path, with expanded ~ sign
-              * exists: is False when the folder did not exist before we tested
-                        whether we can create it
+        * path: is the full path, with expanded ~ sign
+        * exists: is False when the folder did not exist before we tested whether we can create it
+
     """
     logging.info("Location of the 'work' path. Multiple chroots"
                  " (native, device arch, device rootfs) will be created"
@@ -86,9 +84,10 @@ def ask_for_work_path(args):
             # created the work directory or the user has deleted it for
             # whatever reason then we need to write initialize it.
             work_version_file = f"{work}/version"
+            suffix = "-2.x"
             if not os.path.isfile(work_version_file):
                 with open(work_version_file, "w") as handle:
-                    handle.write(f"{pmb.config.work_version}\n")
+                    handle.write(f"{pmb.config.work_version}{suffix}\n")
 
             # Create cache_git dir, so it is owned by the host system's user
             # (otherwise pmb.helpers.mount.bind would create it as root)
@@ -100,17 +99,22 @@ def ask_for_work_path(args):
 
 
 def ask_for_channel(args):
-    """ Ask for the postmarketOS release channel. The channel dictates, which
-        pmaports branch pmbootstrap will check out, and which repository URLs
-        will be used when initializing chroots.
-        :returns: channel name (e.g. "edge", "v21.03") """
+    """Ask for the postmarketOS release channel.
+    The channel dictates, which pmaports branch pmbootstrap will check out,
+    and which repository URLs will be used when initializing chroots.
+
+    :returns: channel name (e.g. "edge", "v21.03")
+    """
     channels_cfg = pmb.helpers.git.parse_channels_cfg(args)
     count = len(channels_cfg["channels"])
 
     # List channels
     logging.info("Choose the postmarketOS release channel.")
     logging.info(f"Available ({count}):")
-    for channel, channel_data in channels_cfg["channels"].items():
+    # Only show the first 3 releases. This includes edge, the latest supported
+    # release plus one. Should be a good solution until new needs arrive when
+    # we might want to have a custom channels.cfg attribute.
+    for channel, channel_data in list(channels_cfg["channels"].items())[:3]:
         logging.info(f"* {channel}: {channel_data['description']}")
 
     # Default for first run: "recommended" from channels.cfg
@@ -146,19 +150,22 @@ def ask_for_ui(args, info):
                 ui_list.pop(i)
                 hidden_ui_count += 1
 
+    # Get default
+    default = args.ui
+    if default not in dict(ui_list).keys():
+        default = pmb.config.defaults["ui"]
+
     logging.info(f"Available user interfaces ({len(ui_list) - 1}): ")
     ui_completion_list = []
     for ui in ui_list:
         logging.info(f"* {ui[0]}: {ui[1]}")
         ui_completion_list.append(ui[0])
     if hidden_ui_count > 0:
-        logging.info(f"NOTE: {hidden_ui_count} user interfaces are not"
-                     " available. If device supports GPU acceleration,"
-                     " set \"deviceinfo_gpu_accelerated\" to make UIs"
-                     " available. See: <https://wiki.postmarketos.org/wiki/"
-                     "Deviceinfo_reference")
+        logging.info(f"NOTE: {hidden_ui_count} UIs are hidden because"
+                     " \"deviceinfo_gpu_accelerated\" is not set (see"
+                     " https://postmarketos.org/deviceinfo).")
     while True:
-        ret = pmb.helpers.cli.ask("User interface", None, args.ui, True,
+        ret = pmb.helpers.cli.ask("User interface", None, default, True,
                                   complete=ui_completion_list)
         if ret in dict(ui_list).keys():
             return ret
@@ -183,6 +190,29 @@ def ask_for_ui_extras(args, ui):
                                    default=args.ui_extras)
 
 
+def ask_for_systemd(args, ui):
+    if "systemd" not in pmb.config.pmaports.read_config_repos(args):
+        return args.systemd
+
+    if pmb.helpers.ui.check_option(args, ui, "pmb:systemd-never"):
+        logging.info("Based on your UI selection, OpenRC will be used as init"
+                     " system. This UI does not support systemd.")
+        return args.systemd
+
+    default_is_systemd = pmb.helpers.ui.check_option(args, ui, "pmb:systemd")
+    not_str = " " if default_is_systemd else " not "
+    logging.info("Based on your UI selection, 'default' will result"
+                 f" in{not_str}installing systemd.")
+
+    choices = pmb.config.allowed_values["systemd"]
+    answer = pmb.helpers.cli.ask("Install systemd?",
+                                 choices,
+                                 args.systemd,
+                                 validation_regex=f"^({'|'.join(choices)})$",
+                                 complete=choices)
+    return answer
+
+
 def ask_for_keymaps(args, info):
     if "keymaps" not in info or info["keymaps"].strip() == "":
         return ""
@@ -228,9 +258,7 @@ def ask_for_timezone(args):
 
 
 def ask_for_provider_select(args, apkbuild, providers_cfg):
-    """
-    Ask for selectable providers that are specified using "_pmb_select"
-    in a APKBUILD.
+    """Ask for selectable providers that are specified using "_pmb_select" in a APKBUILD.
 
     :param apkbuild: the APKBUILD with the _pmb_select
     :param providers_cfg: the configuration section with previously selected
@@ -285,8 +313,7 @@ def ask_for_provider_select(args, apkbuild, providers_cfg):
 
 
 def ask_for_provider_select_pkg(args, pkgname, providers_cfg):
-    """
-    Look up the APKBUILD for the specified pkgname and ask for selectable
+    """Look up the APKBUILD for the specified pkgname and ask for selectable
     providers that are specified using "_pmb_select".
 
     :param pkgname: name of the package to search APKBUILD for
@@ -302,12 +329,14 @@ def ask_for_provider_select_pkg(args, pkgname, providers_cfg):
 
 
 def ask_for_device_kernel(args, device):
-    """
-    Ask for the kernel that should be used with the device.
+    """Ask for the kernel that should be used with the device.
 
     :param device: code name, e.g. "lg-mako"
+
     :returns: None if the kernel is hardcoded in depends without subpackages
+
     :returns: kernel type ("downstream", "stable", "mainline", ...)
+
     """
     # Get kernels
     kernels = pmb.parse._apkbuild.kernels(args, device)
@@ -343,54 +372,15 @@ def ask_for_device_kernel(args, device):
     return ret
 
 
-def ask_for_device_nonfree(args, device):
-    """
-    Ask the user about enabling proprietary firmware (e.g. Wifi) and userland
-    (e.g. GPU drivers). All proprietary components are in subpackages
-    $pkgname-nonfree-firmware and $pkgname-nonfree-userland, and we show the
-    description of these subpackages (so they can indicate which peripherals
-    are affected).
-
-    :returns: answers as dict, e.g. {"firmware": True, "userland": False}
-    """
-    # Parse existing APKBUILD or return defaults (when called from test case)
-    apkbuild_path = pmb.helpers.devices.find_path(args, device, 'APKBUILD')
-    ret = {"firmware": args.nonfree_firmware,
-           "userland": args.nonfree_userland}
-    if not apkbuild_path:
-        return ret
-    apkbuild = pmb.parse.apkbuild(apkbuild_path)
-
-    # Only run when there is a "nonfree" subpackage
-    nonfree_found = False
-    for subpackage in apkbuild["subpackages"].keys():
-        if subpackage.startswith(f"device-{device}-nonfree"):
-            nonfree_found = True
-    if not nonfree_found:
-        return ret
-
-    # Short explanation
-    logging.info("This device has proprietary components, which trade some of"
-                 " your freedom with making more peripherals work.")
-    logging.info("We would like to offer full functionality without hurting"
-                 " your freedom, but this is currently not possible for your"
-                 " device.")
-
-    # Ask for firmware and userland individually
-    for type in ["firmware", "userland"]:
-        subpkgname = f"device-{device}-nonfree-{type}"
-        subpkg = apkbuild["subpackages"].get(subpkgname, {})
-        if subpkg is None:
-            raise RuntimeError("Cannot find subpackage function for "
-                               f"{subpkgname}")
-        if subpkg:
-            logging.info(f"{subpkgname}: {subpkg['pkgdesc']}")
-            ret[type] = pmb.helpers.cli.confirm(args, "Enable this package?",
-                                                default=ret[type])
-    return ret
-
-
 def ask_for_device(args):
+    """
+    Prompt for the device vendor, model, and kernel.
+
+    :returns: Tuple consisting of: (device, device_exists, kernel)
+        * device: "<vendor>-<codename>" string for device
+        * device_exists: bool indicating if device port exists in repo
+        * kernel: type of kernel (downstream, etc)
+    """
     vendors = sorted(pmb.helpers.devices.list_vendors(args))
     logging.info("Choose your target device vendor (either an "
                  "existing one, or a new one for porting).")
@@ -415,9 +405,9 @@ def ask_for_device(args):
             if not pmb.helpers.cli.confirm(args, default=True):
                 continue
         else:
-            # Unmaintained devices can be selected, but are not displayed
+            # Archived devices can be selected, but are not displayed
             devices = sorted(pmb.helpers.devices.list_codenames(
-                args, vendor, unmaintained=False))
+                args, vendor, archived=False))
             # Remove "vendor-" prefixes from device list
             codenames = [x.split('-', 1)[1] for x in devices]
             logging.info(f"Available codenames ({len(codenames)}): " +
@@ -448,17 +438,16 @@ def ask_for_device(args):
             logging.info("Generating new aports for: {}...".format(device))
             pmb.aportgen.generate(args, f"device-{device}")
             pmb.aportgen.generate(args, f"linux-{device}")
-        elif "/unmaintained/" in device_path:
+        elif "/archived/" in device_path:
             apkbuild = f"{device_path[:-len('deviceinfo')]}APKBUILD"
-            unmaintained = pmb.parse._apkbuild.unmaintained(apkbuild)
-            logging.info(f"WARNING: {device} is unmaintained: {unmaintained}")
+            archived = pmb.parse._apkbuild.archived(apkbuild)
+            logging.info(f"WARNING: {device} is archived: {archived}")
             if not pmb.helpers.cli.confirm(args):
                 continue
         break
 
     kernel = ask_for_device_kernel(args, device)
-    nonfree = ask_for_device_nonfree(args, device)
-    return (device, device_exists, kernel, nonfree)
+    return (device, device_exists, kernel)
 
 
 def ask_for_additional_options(args, cfg):
@@ -620,15 +609,36 @@ def ask_build_pkgs_on_install(args):
                                    default=args.build_pkgs_on_install)
 
 
+def get_locales():
+    ret = []
+    list_path = f"{pmb.config.pmb_src}/pmb/data/locales"
+    with open(list_path, "r") as handle:
+        for line in handle:
+            ret += [line.rstrip()]
+    return ret
+
+
 def ask_for_locale(args):
-    locales = pmb.config.locales
-    logging.info(f"Available locales ({len(locales)}): {', '.join(locales)}")
-    return pmb.helpers.cli.ask("Choose default locale for installation",
-                               choices=None,
-                               default=args.locale,
-                               lowercase_answer=False,
-                               validation_regex="|".join(locales),
-                               complete=locales)
+    locales = get_locales()
+    logging.info("Choose your preferred locale, like e.g. en_US. Only UTF-8"
+                 " is supported, it gets appended automatically. Use"
+                 " tab-completion if needed.")
+
+    while True:
+        ret = pmb.helpers.cli.ask("Locale",
+                                  choices=None,
+                                  default=args.locale.replace(".UTF-8", ""),
+                                  lowercase_answer=False,
+                                  complete=locales)
+        ret = ret.replace(".UTF-8", "")
+        if ret not in locales:
+            logging.info("WARNING: this locale is not in the list of known"
+                         " valid locales.")
+            if pmb.helpers.cli.ask() != "y":
+                # Ask again
+                continue
+
+        return f"{ret}.UTF-8"
 
 
 def frontend(args):
@@ -664,11 +674,9 @@ def frontend(args):
         pmb.config.pmaports.install_githooks(args)
 
     # Device
-    device, device_exists, kernel, nonfree = ask_for_device(args)
+    device, device_exists, kernel = ask_for_device(args)
     cfg["pmbootstrap"]["device"] = device
     cfg["pmbootstrap"]["kernel"] = kernel
-    cfg["pmbootstrap"]["nonfree_firmware"] = str(nonfree["firmware"])
-    cfg["pmbootstrap"]["nonfree_userland"] = str(nonfree["userland"])
 
     info = pmb.parse.deviceinfo(args, device)
     apkbuild_path = pmb.helpers.devices.find_path(args, device, 'APKBUILD')
@@ -682,11 +690,16 @@ def frontend(args):
 
     cfg["pmbootstrap"]["user"] = ask_for_username(args)
     ask_for_provider_select_pkg(args, "postmarketos-base", cfg["providers"])
+    ask_for_provider_select_pkg(args, "postmarketos-base-ui", cfg["providers"])
 
     # UI and various build options
     ui = ask_for_ui(args, info)
     cfg["pmbootstrap"]["ui"] = ui
     cfg["pmbootstrap"]["ui_extras"] = str(ask_for_ui_extras(args, ui))
+
+    # systemd
+    cfg["pmbootstrap"]["systemd"] = ask_for_systemd(args, ui)
+
     ask_for_provider_select_pkg(args, f"postmarketos-ui-{ui}",
                                 cfg["providers"])
     ask_for_additional_options(args, cfg)
@@ -733,8 +746,4 @@ def frontend(args):
         # Do not zap any existing packages or cache_http directories
         pmb.chroot.zap(args, confirm=False)
 
-    logging.info("WARNING: The chroots and git repositories in the work dir do"
-                 " not get updated automatically.")
-    logging.info("Run 'pmbootstrap status' once a day before working with"
-                 " pmbootstrap to make sure that everything is up-to-date.")
     logging.info("DONE!")
